/* Versión electrónica ampliada de los ejemplos para SuperCollider asociados al Libro "Composición algorítmica: Matemáticas y ciencias de la computación en la creación musical" del Dr. Jorge Variego, realizados por su autor con contribuciones de Paul Koonce, Paul Berg, Julian Peterson y del SuperCollider book de Nick Collins.
Un precedente de la idea de uso de las reflexiones iniciales de un recinto para la generación de estructuras temporales (Capítulo 8) se encuentra en el trabajo de Pablo Cetta titulado "Un modelo para la simulación del espacio en Música", Editorial Educa, Buenos Aires, 2010. */

(
// El texto "entrena" a la matriz

var text, trainer, states, transitionWeights, segmentedKeys,
	currentKey, currentSegmentedKey, currentKeySymbol,
	maxKeyLength, markovValue, markovText, globalMarkovOrder, printWeights,
	translator ;

trainer = { arg trainingArray, markovOrder ;

			var possibleListOfStates  ;

			globalMarkovOrder = markovOrder ;

		possibleListOfStates = Array.new( trainingArray.size ) ;

		trainingArray.do({ arg character ;

			if( possibleListOfStates.includes( character ).not, {
				possibleListOfStates.add( character )
			});
		});

		states = possibleListOfStates ;

		if( markovOrder == 0, {

			transitionWeights = Array.fill(states.size, { 0.0} );
			trainingArray.do({arg character;
					transitionWeights.put( states.indexOf(character),
						transitionWeights.at( states.indexOf(character) ) + 1
					);

			});
		},{

			transitionWeights = IdentityDictionary.new( trainingArray.size );
			segmentedKeys = IdentityDictionary.new( trainingArray.size );

			trainingArray.size.do({arg index; var currentKey, followingValue,
					followingValueStatesIndex ;

				currentKey = String.new ;
				currentSegmentedKey = Array.new( markovOrder );
				markovOrder.do({arg i;
					currentKey =
						currentKey ++ trainingArray.wrapAt(index + i).asString ;
					if( i != (markovOrder - 1), {currentKey = currentKey ++ ','});
					currentSegmentedKey.add( trainingArray.wrapAt( index + i) ) ;

				});
				currentKeySymbol =  currentKey.asSymbol ;

				if( transitionWeights.includesKey( currentKeySymbol ).not,{
					transitionWeights.add( currentKeySymbol -> Array.fill(states.size, { 0.0 }) );
					segmentedKeys.add(currentKeySymbol -> currentSegmentedKey );
				});

				followingValue = trainingArray.wrapAt( index + markovOrder ) ;
				followingValueStatesIndex = states.indexOf( followingValue ) ;
				transitionWeights.at( currentKeySymbol ).put( followingValueStatesIndex,
					transitionWeights.at( currentKeySymbol ).at( followingValueStatesIndex ) + 1
				);
			});

		});
} ;

markovValue = Routine.new({
	var outValue, lastValue, first=true, currentKey, currentKeySymbol, currentSegmentedKey,
		tempKey, sum ;

		loop{
			if( globalMarkovOrder == 0, {
				outValue = states.wchoose( transitionWeights.normalizeSum );
			},{
				if( first, {
					sum = 0 ;
					transitionWeights.keys.do({arg keySymbol; var sumNow ;
						sumNow = transitionWeights.at( keySymbol ).sum ;
						if( sumNow > sum, {
							currentSegmentedKey = segmentedKeys.at( keySymbol );
							currentKeySymbol = keySymbol ;
							sum = sumNow ;
						});
					});
					first = false ;

				}) ;

				outValue = states.wchoose( transitionWeights.at(currentKeySymbol).normalizeSum );
				currentKey = String.new ;

				if( globalMarkovOrder > 1, {
					for(1, currentSegmentedKey.size - 1, { arg i;
							currentSegmentedKey.put( i - 1, currentSegmentedKey.at(i) );
							currentKey = currentKey ++ currentSegmentedKey.at(i).asString ;
							currentKey = currentKey ++ ',' ;
					});
	 				});
				currentSegmentedKey.put( currentSegmentedKey.size - 1, outValue );
				currentKey = currentKey ++ outValue.asString ;
	 			currentKeySymbol = currentKey.asSymbol ;


			}) ;

			outValue.yield ;

		};
}) ;

text=
"Cantando me he de morir
Cantando me han de enterrar,
Y cantando he de llegar
Al pie del eterno padre:
Dende el vientre de mi madre
Vine a este mundo a cantar."  ;

text =  Array.fill( text.size, {arg c; text[c].asSymbol });

trainer.value( text, 5 ) ;

printWeights.value ;

text = "" ;
160.do({ text = text ++ markovValue.next.asString });

text.postln;
)