/* Versión electrónica ampliada de los ejemplos para SuperCollider asociados al Libro "Composición algorítmica: Matemáticas y ciencias de la computación en la creación musical" del Dr. Jorge Variego, realizados por su autor con contribuciones de Paul Koonce, Paul Berg, Julian Peterson y del SuperCollider book de Nick Collins.
Un precedente de la idea de uso de las reflexiones iniciales de un recinto para la generación de estructuras temporales (Capítulo 8) se encuentra en el trabajo de Pablo Cetta titulado "Un modelo para la simulación del espacio en Música", Editorial Educa, Buenos Aires, 2010. */

/// Manejo de la forma con un criterio aleatorio controlado CON sonido

(

SynthDef(\campanas1, {
	arg freqRing=100;
    var freqs, ringtimes, signal;
    freqs = Control.names([\freqs]).kr([800, 1071, 1153, 1723]);
    ringtimes = Control.names([\ringtimes]).kr([1, 1, 1, 1]);
	signal = Limiter.ar(DynKlank.ar(`[freqs, nil, ringtimes ], Impulse.ar(freqRing, 0, 0.1)), 0.5);
    Out.ar(0, signal!2);
}).add;

)

(
var totalDur = 30, units; // Duración total puede ser decidida en forma anticipada, el resto de las decisiones son delegadas en el algoritmo.

units =(((Array.fill(rrand(3, 7), {|i| rand(100)})).sort.reverse.normalizeSum)) * totalDur; // Genera una coleccion de entre 3 y 8 valores aleatorios entre 0 - 100 que son el tiempo de espera entre cada sección nueva de la pieza. En este ejmplo son ordenados en forma de "accelerando", desde las más largas a las más cortas.

a = Synth(\campanas1);

r = Routine{

	("Esta obra tendrá: " ++ units.size ++ " secciones").postln;
	(units.size).do{ |i|

	("Sección: " ++ (i+1) ++ " Duracion: " ++ (units[i]) ++ " segundos").postln;

		// En cada sección de la obra, los parámetros del instrumento cambian

		a.setn(\freqs, Array.rand(4, 60, 2000)); // Cuatro parciales del instrumento campanas1
		a.setn(\ringtimes, Array.rand(4, 2, 5) ); // Tiempos de resonancia para cada uno de los parciales
		a.set(\freqRing, (i + 1)); // Tiempo de los impulsos (tambien accelerando. A medida que avanza la forma, hay mas impulsos por unidad de tiempo

	units[i].wait;

	};

"****************FIN".postln;

}.play;

)


