/* Versión electrónica ampliada de los ejemplos para SuperCollider asociados al Libro "Composición algorítmica: Matemáticas y ciencias de la computación en la creación musical" del Dr. Jorge Variego, realizados por su autor con contribuciones de Paul Koonce, Paul Berg, Julian Peterson y del SuperCollider book de Nick Collins.
Un precedente de la idea de uso de las reflexiones iniciales de un recinto para la generación de estructuras temporales (Capítulo 8) se encuentra en el trabajo de Pablo Cetta titulado "Un modelo para la simulación del espacio en Música", Editorial Educa, Buenos Aires, 2010. */

(
// Instrumento basado en 24 parciales sobre una frecuEncia dada. Las amplitudes tienen un envolvelnte propio.
SynthDef(\test, {
	arg out = 0, baseFrec = 110, ratio = 1.0, release=2, amp=1, gate=1, duration = 4;
	var env1 = Env([0.0, 0.9, 0.0], [0.5, 0.4], -4);
	var env2 = Env([0.0, 0.9, 0.0], [0.9, 0.1], -2);
	var envolventes = [env1, env2];
	var gen1 = EnvGen.kr(env1, timeScale: duration, doneAction: 2);
	var gen2 = EnvGen.kr(env2, timeScale: duration, doneAction: 2);
	var amplitudes = (Array.fill(24, {|i| 1/(i+1) ;}));
	var sound = Array.fill(24, { |i | SinOsc.ar((baseFrec * (i + (1 * (ratio * rrand(1.0, 1.2))))), amplitudes[i] * gen2)}); // El instrumento tiene incorporado un elemento aleatorio en la configuracion del espectro
	var mixSound = Limiter.ar(Mix.ar(sound), 0.7);
	Out.ar(out, mixSound!2 * gen1);

}).add;
)
(
// Instrumento que solo "toca" las frecuencias base que permanecen inamovibles.
SynthDef(\sine, {
	arg out = 0, baseFrec = 110, ratio = 1.0, release=2, amp=1, gate=1, duration = 4;
	var env1 = Env([0.0, 0.9, 0.0], [0.5, 0.4], -4);
	var gen1 = EnvGen.kr(env1, timeScale: duration, doneAction: 2);
	Out.ar(out, SinOsc.ar(baseFrec) * gen1 !2);

}).add;
)
( // Esta rutina interpreta el fragmento del preludio de Bach con un espectro que cambia gradualmente en forma independiente de la lineas melodica que conforma las frecuencias base.
Routine({

	var tempo = 0.1, overlap=tempo/2;
	var dev=1.0;
	var sobre = [60, 64, 67, 72, 76, 67, 72, 76, 60, 64, 67, 72, 76, 67, 72, 76, 60, 62, 69, 74, 77, 69, 74, 77 , 60, 62, 69, 74, 77, 69, 74, 77, 59, 62, 67, 74, 77, 67, 74, 77, 59, 62, 67, 74, 77, 67, 74, 77];
	var melodia = Pseq(sobre, inf).asStream;
	var melodia1 = Pseq(sobre, inf).asStream;// Primeras notas del preludio en Do mayor de J.S Bach que sirven como frecuencias base para el cálculo del espectro de cada Synth. El componente ratio es el multiplicador que permite controlar la relación entre los parciales.
inf.do{

sobre.size.do{

		Synth(\sine,[\baseFrec, melodia1.next.midicps, \release, tempo + overlap, \duration, tempo*2]);
		Synth(\test,[\baseFrec, melodia.next.midicps, \release, tempo + overlap, \ratio, dev, \duration, tempo*4]);
		tempo.wait;

	};

		dev = dev + 0.5; // Desviación acumulativa
		("Desviación inarmónica: " ++ dev).postln;

	};

};).play;
)


