/* Versión electrónica ampliada de los ejemplos para SuperCollider asociados al Libro "Composición algorítmica: Matemáticas y ciencias de la computación en la creación musical" del Dr. Jorge Variego, realizados por su autor con contribuciones de Paul Koonce, Paul Berg, Julian Peterson y del SuperCollider book de Nick Collins.
Un precedente de la idea de uso de las reflexiones iniciales de un recinto para la generación de estructuras temporales (Capítulo 8) se encuentra en el trabajo de Pablo Cetta titulado "Un modelo para la simulación del espacio en Música", Editorial Educa, Buenos Aires, 2010. */

(
SynthDef("pulseWaves", { arg base = 80, range = 0,
				duration = 1.0, gate = 1,
				reciprocalQ = 0.01, dbGain = 24,
					vowelLagTime = 0  ;

	var controlSine1, controlSine2, audioPulseWave, resonLPF,
		cSine2FreqVals, combDelayTimes, output  ;
	var multiplier, adder ;
	var env, envSig ;
	var vowel, vowelFilteredAudio, vowelMix ;
	var cfs ;

	cfs = Control.names(\cfs).kr([ 300, 870, 2240 ]) ;

	env = Env.asr( 0.1, 1.0, 0.5, 1.0, 'linear' )  ;

	envSig = EnvGen.kr( env, timeScale: duration, gate: gate, doneAction: 2 ) ;

	multiplier = range / 2.0 ;
	adder = base + multiplier ;

	controlSine1 = FSinOsc.kr(0.05, 0, multiplier, adder ) ;

	audioPulseWave = Pulse.ar(controlSine1, 0.4, 0.2 * (dbGain.dbamp)) ;

	vowel = Lag.kr( cfs, vowelLagTime ) ;

	vowelFilteredAudio = BPF.ar(audioPulseWave, vowel, reciprocalQ ) ;
	output = Mix.ar( vowelFilteredAudio ) ;



	combDelayTimes = [0.2,0.25] ;
/output = CombL.ar(vowelMix, 0.3, combDelayTimes, 2)  ; // ARRAY OF TWO SIGNALS

	Out.ar( 0, output * envSig ) ;

	Out.ar( 0, [vowelMix] * envSig ) ;



} ).send(s) ;
)

(
var r, base, range, totalDur ;

totalDur = 20 ;

r = Routine.new({

	var vowels, vowelNow, vowelArrayOfCFsNow, nextTime, waitTime, midiPitch,
		topLineMIDIpitches=(60 + [ 0, 2, 2, 1, 2, 2, 2, 1 ].integrate),
		bottomLineMIDIpitches=[[57, 55, 53, 52], [56, 54, 53], [56, 54, 53],[56, 54, 53],[56, 54, 53],[56, 54, 53],[56, 54, 53],[56, 54, 53]],      //// PLEASE CONTINUE
			newTopLineIndex, lastTopLineIndex, leapFlag=0, lowVoice, bottomLineMIDIpitchesNow,
				intervalArray=[0, 3, 4, 7, 9, 12 ], interval, bottomMidiPitch ;

	vowels = IdentityDictionary[
	\uh -> [	300, 870, 2240 ],
	\ah -> [   730, 1090, 2440 ],
	\eh -> [ 390, 1990, 2550],
	\eee -> [ 270, 2290, 3010 ]
	];

	z = Synth( "pulseWaves" ) ;

	lowVoice = Synth( "pulseWaves" ) ;

	nextTime = 0 ;

	waitTime = 2 ;
	nextTime = nextTime  + waitTime ;

	lastTopLineIndex = topLineMIDIpitches.size.rand ;

	midiPitch = 60 ;


	z.set( \range, 0.0, \vowelLagTime, 0.2, \base, 60.midicps );

	lowVoice.set( \range, 0.0, \vowelLagTime, 0.2, \base, 60.midicps );


	while( {nextTime < totalDur}, {
		vowelNow = vowels.keys.choose ;

		("Singing: \t" ++ vowelNow ).postln;
		vowelArrayOfCFsNow = vowels.at( vowelNow ) ; // vowels[]

		if( [true, false].wchoose( [7, 0].normalizeSum ), {

			if( leapFlag == 0, {
				// NO LEAP
				if( [true, false].wchoose( [7, 3].normalizeSum ), {
					// STEPWISE
					newTopLineIndex = lastTopLineIndex + [-1, 1].choose ;
				},{
					// LEAP
					newTopLineIndex = lastTopLineIndex +
							( [-1, 1].choose * rrand(2, topLineMIDIpitches.size - 1) ) ;
					"LEAPING...................!".postln ; 				leapFlag = -1 * (newTopLineIndex - lastTopLineIndex).sign ;
				}) ;
			},{
				// IT LEPT
				newTopLineIndex = lastTopLineIndex + leapFlag ;
					leapFlag = 0 ;
			});

			// KEEP INDEX IN ARRAY RANGE
			if( newTopLineIndex >= topLineMIDIpitches.size, {
				newTopLineIndex = topLineMIDIpitches.size - 2 ;
			});
			if( newTopLineIndex <= 0, {
				newTopLineIndex = 1 ;
			});

			midiPitch = topLineMIDIpitches.at( newTopLineIndex ) ;
			bottomLineMIDIpitchesNow = bottomLineMIDIpitches[newTopLineIndex ].choose;
			z.set( \base, midiPitch.midicps  );
			lowVoice.set( \base, bottomLineMIDIpitchesNow.midicps  );
			lastTopLineIndex = newTopLineIndex ;

		});


		z.setn( \cfs, vowelArrayOfCFsNow ) ;

		waitTime.wait;
		waitTime = [1, 0.5].wchoose( [3, 6].normalizeSum ) ;
		nextTime = nextTime  + waitTime ;

	});

	// Wait half second before saying we're done

	z.set( \gate, 0.0, \duration, 5.0 ) ;
	lowVoice.set( \gate, 0.0, \duration, 5.0 ) ;

	"I'm waiting on the final decay......".postln ;
//	7.0.wait;

	"done".postln;
});


SystemClock.play(r);
)

