/* Versión electrónica ampliada de los ejemplos para SuperCollider asociados al Libro "Composición algorítmica: Matemáticas y ciencias de la computación en la creación musical" del Dr. Jorge Variego, realizados por su autor con contribuciones de Paul Koonce, Paul Berg, Julian Peterson y del SuperCollider book de Nick Collins.
Un precedente de la idea de uso de las reflexiones iniciales de un recinto para la generación de estructuras temporales (Capítulo 8) se encuentra en el trabajo de Pablo Cetta titulado "Un modelo para la simulación del espacio en Música", Editorial Educa, Buenos Aires, 2010. */

(

//// Instrumento

SynthDef.new(\instrumento, {
	arg buf=0, semitoneTranspose=1.0, amp=1, pan=0.0, envTimeScale=1.0, out;
	var sig, envsig, env, rate;
	rate = (60 + semitoneTranspose).midicps / 60.midicps ;
	sig = PlayBuf.ar(1, buf, BufRateScale.ir(buf) * rate);
	env  = Env.new( [ 0.0, 1.0, 1.0, 0.0 ],[ 0.001, 0.699, 0.3 ],'linear');
	envsig = EnvGen.kr( env, timeScale: envTimeScale, doneAction: 2 ) ;
	sig = Pan2.ar(sig, pan, amp);
	Out.ar(out, sig * envsig);
}).add;

//// Sonido

~diccionario = Dictionary.new; // Diccionario de sonidos

~diccionario.add(\vibes ->
	PathName("/Users/musicstaff/Misonido").entries.collect({
		arg sf;
		Buffer.read(s, sf.fullPath); //
	});

);

)

//// Pattern (puede evaluarse en forma recurrente)

(

~tempo = 1.1; //

Pdef(
	\,
	Pbind(
		\instrument, \instrumento,
		\dur, (Pseq([1/32, 2/32, 3/32, 4/32, 5/32, 6/32, 7/32, 8/32, 9/32, 10/32, 11/32, 12/32], inf) * ~tempo), // Serie de duraciones acumulativas
		\stretch, 2,
		\buf, Prand(~diccionario[\vibes], inf), // Referimos al diccionario de sonidos
		\rate, 1,
		\semitoneTranspose, Pseq([3, 2, 9, 8, 7, 6, 4, 1, 0, 10, 5, 11], inf) , // Serie dodecafónica principal
		\amp, Pseq([1/12, 2/12, 3/12, 4/12, 5/12, 6/12, 7/12, 8/12, 9/12, 10/12, 11/12, 12/12], inf), // Dinámicas
		\pan, Pseq([-1, -1 + (0.181 * 1), -1 + (0.181 * 2), -1 + (0.181 * 3), -1 + (0.181 * 4), -1 + (0.181 * 5), -1 + (0.181 * 6), -1 + (0.181 * 7), -1 + (0.181 * 8), -1 + (0.181 * 9), -1 + (0.181 * 10), -1 + (0.181 * 11)], inf), // El paneo esta serializado en 12 entre -1 (izquierda) y 1 (derecha).
		\envTimeScale, 3,
		\out, 0,
	);

).play(quant: 2); //quant_(0.5); // Se puede cuantizar con otro Pdef

)

